/*
 * rssyslog
 * The beginnings of what is to grow into a major syslog-server
 * Copyright (C) 2005-2007 Rune Stre
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Rune Stre <rst@folesvaert.no>
 *  
 */

/*
 * message.h
 * Data structures and functions for working on syslog messages
 */


#ifndef rssyslog_message_h
#define rssyslog_message_h

//  #include <sys/socket.h>
//  #include <sys/uio.h>
#include <netinet/in.h>
#include <time.h>

#define DATE_STRLEN 26
#define MSG_MAXLEN 1024

enum AddressType { IPv4_UDP };

enum PartNames {
  pLocalTS = 0, /* local timestamp */
  pLocalSep1,
  pLocalHost,   /* Source IP of host */
  pLocalSep2,
  pTS,          /* timestamp from message */
  pSep1, 
  pHost,        /* hostname from message */ 
  pSep2, 
  pFac,         /* Facility */
  pSep3,
  pSev,         /* Severity */
  pSep4,
  pMsg,         /* the rest of the message */
  pNL,          /* trailing newline */ 
  pNumParts     /* Total number of parts */
};   


struct Message {
  char buf[MSG_MAXLEN];
  int buflen;

  enum AddressType addr_type;
  socklen_t addr_len;
  union {
    struct sockaddr_in ipv4_udp;
  } addr;

  struct timeval ts;
  int msg_nr;

  uint32_t src_ip;
  struct tm date;

  struct iovec parts[pNumParts];
  int parts_used;

  int fac;
  int sev;

  char ip_str[INET_ADDRSTRLEN];
  char date_str[DATE_STRLEN+1];

};




#endif /* rssyslog_decode_message_h */
