/*
 * rssyslog
 * The beginnings of what is to grow into a major syslog-server
 * Copyright (C) 2007 Rune Stre
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Rune Stre <rst@folesvaert.no>
 *  
 */

/*
 * msg_queue.h
 * Header file for message queuing code.
 */

#ifndef rssyslog_msg_queue_h
#define rssyslog_msg_queue_h

#include "message.h"
#include <pthread.h>

/* from init_targets.h */
struct Config;

struct MsgQueue {
  
  pthread_mutex_t lock;
  pthread_cond_t  cond_full;
  pthread_cond_t  cond_empty;

  int n;
  int wr_head;
  int wr_tail;

  int rd_head;
  int rd_tail;

  int n_full;
  int n_empty;

  int bailout;

  struct Message* q;

};

/*
 * Create and delete a queue
 */
struct MsgQueue* msgqueue_init(const struct Config* c);
int msgqueue_destroy(struct MsgQueue* q); /* Noone should be using the queue */


/*
 * Signal all clients to quit
 */

int msgqueue_bailout(struct MsgQueue* q);

/* 
 * Methods used by the queue writer
 * Messages are returned to the queue in the same order they are acquired.
 * msgqueue_writer_get() returns NULL when msgqueue_bailout() is called.
 */
struct Message* msgqueue_writer_get(struct MsgQueue* q);
struct Message* msgqueue_writer_get_nowait(struct MsgQueue* q);
int             msgqueue_writer_release(struct MsgQueue* q);

/* 
 * Methods used by the queue reader
 * Messages are returned to the queue in the same order they are acquired.
 * msgqueue_reader_get() returns NULL when msgqueue_bailout() is called.
 */
struct Message* msgqueue_reader_get(struct MsgQueue* q);
struct Message* msgqueue_reader_get_nowait(struct MsgQueue* q);
int             msgqueue_reader_release(struct MsgQueue* q);




#endif /* rssyslog_msg_queue_h */

