/*
 * rssyslog
 * The beginnings of what is to grow into a major syslog-server
 * Copyright (C) 2005-2007 Rune Stre
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Rune Stre <rst@folesvaert.no>
 *  
 */

/*
 * receive_message.c
 * Implementation of functions for setting up listening sockets 
 * and receiving messages.
 */

#include "receive_message.h"
#include "decode_message.h"
#include "msg_queue.h"
#include "rssyslogd.h" /* For the warn() anad fail() functions */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <sys/socket.h> /* socket, */
#include <netinet/in.h>
/*  #include <arpa/inet.h> */
/*  #include <netdb.h> */
#include <errno.h>   /* errno */
#include <string.h>  /* memset,  */
#include <unistd.h>  /* close */

#include <sys/time.h> /* gettimeofday */
#include <time.h>     /* gettimeofday */

#define RECVPORT 514


int listener_receive_ipv4_udp(int sd, struct Message* m)
{
  if (m == NULL)
    return -1;

  m->addr_type = IPv4_UDP;
  m->addr_len = sizeof(m->addr.ipv4_udp);
  m->buflen = recvfrom( sd, m->buf, sizeof(m->buf), 0, 
                        (struct sockaddr *)&(m->addr.ipv4_udp), 
                        &(m->addr_len) );
  if(m->buflen < 0)
    warn(errno, "recvfrom() failed");

  if( gettimeofday(&(m->ts), NULL) < 0)
    warn(errno, "gettimeofday() failed");

  if(m->buflen < 0)
    return -1;

  return 0;
}


int listener_init ( struct ListenerData* l,
                    struct MsgQueue* q, 
                    const struct Config* c)
{
  struct sockaddr_in saddr;

  l->q = q;
  l->c = c;
  l->msg_counter = 0;
  l->bailout = 0;

  l->udp_sd = socket(AF_INET, SOCK_DGRAM, 0);
  if(l->udp_sd < 0)
    fail(errno, "sock_open_ipv4_udp(): socket() failed");

  memset(&saddr, 0, sizeof(saddr));
  saddr.sin_port = htons(RECVPORT);
  saddr.sin_family = AF_INET;

  if (bind(l->udp_sd, (struct sockaddr *)&saddr, sizeof(saddr)) < 0) {
    close(l->udp_sd);
    fail(errno, "sock_open_ipv4_udp(): bind() failed");
  }
 
  return 0;
}

int listener_close(struct ListenerData* l)
{
  close(l->udp_sd);
  l->udp_sd = -1;
  return 0;
}


int listener_thread(void* user_data)
{
  struct ListenerData* d = (struct ListenerData*) user_data;
  struct Message tmp_msg;
  init_message(d->c, &tmp_msg);

  if (d == NULL)
    return -1;
  
  while (d->bailout == 0) {
    
    /* Try to get a message slot. */
    struct Message* m = msgqueue_writer_get_nowait(d->q);
    struct Message* m_cur = m;
    /* Use a static buffer if we didn't get a slot */
    if (m == NULL) {
      m_cur = &tmp_msg;
      reset_message(d->c, m_cur);
    }

    /* Reset the buffer, get next message and increment msg_counter */
    while (listener_receive_ipv4_udp(d->udp_sd, m_cur) < 0) {
      if (d->bailout)
        return 0;
      else
        warn(-1, "udp_listener: Failed to receive message");
    }
    m_cur->msg_nr = d->msg_counter++;

    /* Try once more to get a message slot if we do not have one */
    if (m == NULL)
      if ( (m = msgqueue_writer_get_nowait(d->q)) != NULL)
        memcpy(m, m_cur, sizeof(struct Message));

    /* Return the slot to the queue if appropriate */
    if (m != NULL)
      msgqueue_writer_release(d->q);
    
    /* Done. Loop back */
  }
  return 0;
}

