/*
 * rs_thread
 * C thread handling abstraction
 * Copyright (C) 2001-2007 Rune Stre
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Rune Stre <rst@folesvaert.no>
 *  
 */

/*
 * rs_thread.c
 * This file implements the functions in rs_thread.h
 * 11.2.2001, Rune Saetre 
 */

#include "rs_thread.h"

int rs_thread_init( struct RSThread* t, 
                    int(*thread_function)(void* user_data),
                    void* user_data )
{
  if(t == NULL)
    return -1;

  if(thread_function == NULL)
    {
      t->thread_function = NULL;
      t->_tid = -1;
      return -1;
    }


  t->user_data       = user_data;
  t->thread_function = thread_function;

  t->_tid            = -1;
  t->_return_value   = 0;

  return 0;

}

int rs_thread_start(struct RSThread* t)
{
  if(t == NULL)
    return -1;

  if(t->thread_function == NULL)
    return -1;

  {
    int status = 
      pthread_create( &(t->_tid), NULL, _rs_thread_runner, (void*) t );
    
    if(status != 0)
      {
        t->_tid = -1;
        return -1;
      }
  }
  return 0;
}

int rs_thread_wait(struct RSThread* t)
{
  if(t == NULL)
    return -1;
  
  if(t->_tid == -1)
    return -1;

  {
    int status = pthread_join(t->_tid, NULL);
    if(status != 0)
      return -1;
  }

  /* 
   * The thread has finished. 
   * thread_function's return value should be placed in
   * t->_return_value.
   */

  return t->_return_value;
  
}



void* _rs_thread_runner(void* ptr)
{
  struct RSThread* t = (struct RSThread*) ptr;

  if(t == NULL)
    return NULL;

  if(t->thread_function == NULL)
    return NULL;

  
  t->_return_value = t->thread_function(t->user_data);

  return NULL;
}
