/*
 * rs_thread
 * C thread handling abstraction
 * Copyright (C) 2001-2007 Rune Stre
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Rune Stre <rst@folesvaert.no>
 *  
 */
/*
 * rs_thread.h
 * This file defines a thread abstraction based on PosixThreads
 * 11.2.2001, Rune Saetre
 */

#ifndef THREAD_MANAGER_H
#define THREAD_MANAGER_H

#include <pthread.h>


struct RSThread
{
  /* 
   * User data 
   */
  void* user_data; /* Can be NULL if it is not needed. */
  int (*thread_function)(void* user_data);

  /* 
   * Data for internal use by rs_thread 
   */
  pthread_t _tid;
  int _return_value;
};

/* 
 * Initialize the thread datastructure.
 * user_data can be NULL if it is not needed by the thread.
 * DO NOT CALL THIS ON A RUNNING THREAD!!!
 * Returns 0 on success, -1 on error.
 */
int rs_thread_init( struct RSThread* t, 
                    int(*thread_function)(void* user_data),
                    void* user_data );

/* 
 * Start the thread.
 * Returns 0 on success, -1 on error.
 */
int rs_thread_start(struct RSThread* t);

/* 
 * Wait for the thread to quit.
 * Returns the return value of t->thread_function,
 * or -1 if t does not point to a thread that has been started.
 */
int rs_thread_wait(struct RSThread* t);



/* This function is used internally */
void* _rs_thread_runner(void*);

#endif /* THREAD_MANAGER_H */
