/*
 * rssyslog
 * The beginnings of what is to grow into a major syslog-server
 * Copyright (C) 2005-2007 Rune Stre
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Rune Stre <rst@folesvaert.no>
 *  
 */

/*
 * syslog_names.c
 * Functions for looking up syslog facility and priority names
 */

#include "syslog_names.h"

/* Needed for allocation of facilitynames and prioritynames in sys/syslog.h */
#define SYSLOG_NAMES 1

#include <stdio.h>     /* NULL */
#include <sys/syslog.h>
#include <string.h>    /* strcmp */


char* get_facility_name(int fac)
{
  CODE* n = facilitynames;
  while( (n->c_name) && (n->c_val!=(fac<<3)) )
    n++;

  return n->c_name;
}

char* get_priority_name(int pri)
{
  CODE* n = prioritynames;
  while(  (n->c_name) && (n->c_val!=pri) )
    n++;
  return n->c_name;
}

int get_facility_number(const char* name)
{
  printf("get_facility_number(%s)\n", name);

  CODE* n = facilitynames;
  while ( n->c_name && strcmp(n->c_name, name) )
    n++;
  if (n->c_name == NULL)
    return -1;
  
  return n->c_val >> 3;
}


int get_priority_number(const char* name)
{
  CODE* n = prioritynames;
  while ( n->c_name && strcmp(n->c_name, name) )
    n++;
  if (n->c_name == NULL)
    return -1;

  return n->c_val;
}


